class Messages:
    USER_PROVIDED_CONFIG_HEADER = "All user provided configs for brokers in the cluster are:"


class Errors:
    CONFIG_FILE_SHOULD_BE_PROVIDED = "A configuration file should be provided for connect cluster"



from kfk.commands.main import kfk
from kfk.commands.clusters import clusters
from kfk.commands.topics import topics
from kfk.commands.users import users
from kfk.commands.configs import configs
from kfk.commands.version import version
from kfk.commands.acls import acls
from kfk.commands.operator import operator
from kfk.commands.connect import connect
from kfk.commands.env import env
from kfk.setup import setup

setup()


import click

from kfk.commands.main import kfk
from kfk.commons import print_missing_options_for_command, create_temp_file
from kfk.kubectl_command_builder import Kubectl
from kfk.config import *


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('--uninstall', 'is_uninstall', help='Uninstalls Strimzi Kafka Operator', is_flag=True)
@click.option('--install', 'is_install', help='Installs Strimzi Kafka Operator', is_flag=True)
@kfk.command()
def operator(is_install, is_uninstall, namespace):
    """Installs/Uninstalls Strimzi Kafka Operator"""
    if is_install:
        for directory_name, dirs, files in os.walk("{strimzi_path}/install/cluster-operator/".format(
                strimzi_path=STRIMZI_PATH)):
            for file_name in files:
                file_path = os.path.join(directory_name, file_name)

                if "RoleBinding" in file_name:
                    with open(file_path) as file:
                        stream = file.read().replace("myproject", namespace)
                        temp_file = create_temp_file(stream)
                        file_path = temp_file.name
                os.system(Kubectl().apply().from_file(file_path).namespace(namespace).build())
    elif is_uninstall:
        # TODO: refactor here
        for directory_name, dirs, files in os.walk("{strimzi_path}/install/cluster-operator/".format(
                strimzi_path=STRIMZI_PATH)):
            for file_name in files:
                file_path = os.path.join(directory_name, file_name)

                if "RoleBinding" in file_name:
                    with open(file_path) as file:
                        stream = file.read().replace("myproject", namespace)
                        temp_file = create_temp_file(stream)
                        file_path = temp_file.name
                os.system(Kubectl().delete().from_file(file_path).namespace(namespace).build())
    else:
        print_missing_options_for_command("operator")



import click
import os
import yaml

from kfk.commands.main import kfk
from kfk.option_extensions import NotRequiredIf
from kfk.commons import *
from kfk.kubectl_command_builder import Kubectl
from kfk.config import *


@click.option('-n', '--namespace', help='Namespace to use')
@click.option('--delete-config', help='A cluster configuration override to be removed for an existing cluster',
              multiple=True)
@click.option('--config', help='A cluster configuration override for the cluster being altered.',
              multiple=True)
@click.option('--alter', 'is_alter', help='Alter the cluster.', is_flag=True)
@click.option('--delete', 'is_delete', help='Delete the cluster.', is_flag=True)
@click.option('--create', 'is_create', help='Create the cluster.', is_flag=True)
@click.option('--describe', 'is_describe', help='List details for the given cluster.', is_flag=True)
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--list', 'is_list', help='List all available clusters.', required=True, is_flag=True)
@click.option('--cluster', help='Cluster Name', required=True, cls=NotRequiredIf, not_required_if=['is_list'])
@kfk.command()
def clusters(cluster, is_list, is_create, is_describe, is_delete, is_alter, config, delete_config, output, namespace):
    """Creates, alters, deletes, describes Kafka cluster(s)."""
    if is_list:
        list(namespace)
    elif is_create:
        create(cluster, namespace)
    elif is_describe:
        describe(cluster, output, namespace)
    elif is_delete:
        delete(cluster, namespace)
    elif is_alter:
        alter(cluster, config, delete_config, namespace)
    else:
        print_missing_options_for_command("clusters")


def list(namespace):
    os.system(Kubectl().get().kafkas().namespace(namespace).build())


def create(cluster, namespace):
    with open('{strimzi_path}/examples/kafka/kafka-ephemeral.yaml'.format(strimzi_path=STRIMZI_PATH).format(
            version=STRIMZI_VERSION)) as file:
        stream = file.read().replace('my-cluster', cluster)
        cluster_temp_file = create_temp_file(stream)
        open_file_in_system_editor(cluster_temp_file.name)
        is_confirmed = click.confirm("Are you sure you want to create the cluster with the saved configuration?")
        if is_confirmed:
            os.system(Kubectl().create().from_file("{cluster_temp_file_path}").namespace(namespace).build().format(
                cluster_temp_file_path=cluster_temp_file.name))
        cluster_temp_file.close()


def describe(cluster, output, namespace):
    if output is not None:
        os.system(Kubectl().get().kafkas(cluster).namespace(namespace).output(output).build())
    else:
        os.system(Kubectl().describe().kafkas(cluster).namespace(namespace).build())


def delete(cluster, namespace):
    is_confirmed = click.confirm("Are you sure you want to delete the cluster?")
    if is_confirmed:
        os.system(Kubectl().delete().kafkas(cluster).namespace(namespace).build())


def alter(cluster, config, delete_config, namespace):
    if len(config) > 0 or len(delete_config) > 0:
        stream = get_resource_as_stream("kafkas", cluster, namespace)
        cluster_dict = yaml.full_load(stream)

        delete_last_applied_configuration(cluster_dict)

        if len(config) > 0:
            if cluster_dict["spec"]["kafka"].get("config") is None:
                cluster_dict["spec"]["kafka"]["config"] = {}
            add_resource_kv_config(config, cluster_dict["spec"]["kafka"]["config"])

        if len(delete_config) > 0:
            if cluster_dict["spec"]["kafka"].get("config") is not None:
                delete_resource_config(delete_config, cluster_dict["spec"]["kafka"]["config"])

        cluster_yaml = yaml.dump(cluster_dict)
        cluster_temp_file = create_temp_file(cluster_yaml)
        os.system(
            Kubectl().apply().from_file("{cluster_temp_file_path}").namespace(namespace).build().format(
                cluster_temp_file_path=cluster_temp_file.name))
        cluster_temp_file.close()
    else:
        os.system(Kubectl().edit().kafkas(cluster).namespace(namespace).build())




import click
import os
import yaml

from kfk.commands.main import kfk
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.commons import *
from kfk.config import *
from kfk.kubectl_command_builder import Kubectl
from kfk.constants import *


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--delete-config', help='A topic configuration override to be removed for an existing topic',
              multiple=True)
@click.option('--config', help='A topic configuration override for the topic being created or altered.', multiple=True)
@click.option('--alter', 'is_alter',
              help='Alter the number of partitions, replica assignment, and/or configuration of the topic.',
              is_flag=True)
@click.option('--delete', 'is_delete', help='Delete a topic.', is_flag=True)
@click.option('--command-config',
              help='Property file containing configs to be config property file passed to Admin Client.')
@click.option('--native', help='List details for the given topic natively.', is_flag=True, cls=RequiredIf,
              required_if=['is_describe'])
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--describe', 'is_describe', help='List details for the given topic.', is_flag=True)
@click.option('--replication-factor', help='The replication factor for each partition in the topic being created.',
              cls=RequiredIf, required_if=['is_create'], type=int)
@click.option('--partitions', help='The number of partitions for the topic being created or altered ', cls=RequiredIf,
              required_if=['is_create'], type=int)
@click.option('--create', 'is_create', help='Create a new topic.', is_flag=True)
@click.option('--list', 'is_list', help='List all available topics.', is_flag=True)
@click.option('--topic', help='Topic Name', required=True, cls=NotRequiredIf, not_required_if=['is_list'])
@kfk.command()
def topics(topic, is_list, is_create, partitions, replication_factor, is_describe, output, native, command_config,
           is_delete, is_alter, config, delete_config, cluster, namespace):
    """Creates, alters, deletes, describes Kafka topic(s)."""
    if is_list:
        list(cluster, namespace)
    elif is_create:
        create(topic, partitions, replication_factor, config, cluster, namespace)
    elif is_describe:
        describe(topic, output, native, command_config, cluster, namespace)
    elif is_delete:
        delete(topic, cluster, namespace)
    elif is_alter:
        alter(topic, partitions, replication_factor, config, delete_config, cluster, namespace)
    else:
        print_missing_options_for_command("topics")


def list(cluster, namespace):
    os.system(
        Kubectl().get().kafkatopics().label("strimzi.io/cluster={cluster}").namespace(namespace).build().format(
            cluster=cluster))


def create(topic, partitions, replication_factor, config, cluster, namespace):
    with open('{strimzi_path}/examples/topic/kafka-topic.yaml'.format(strimzi_path=STRIMZI_PATH).format(
            version=STRIMZI_VERSION)) as file:
        topic_dict = yaml.full_load(file)

        topic_dict["metadata"]["name"] = topic
        topic_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster
        topic_dict["spec"]["partitions"] = int(partitions)
        topic_dict["spec"]["replicas"] = int(replication_factor)

        if len(config) > 0:
            if topic_dict["spec"].get("config") is None:
                topic_dict["spec"]["config"] = {}
            add_resource_kv_config(config, topic_dict["spec"]["config"])

        topic_yaml = yaml.dump(topic_dict)
        topic_temp_file = create_temp_file(topic_yaml)
        os.system(
            Kubectl().create().from_file("{topic_temp_file_path}").namespace(namespace).build().format(
                topic_temp_file_path=topic_temp_file.name))
        topic_temp_file.close()


def describe(topic, output, native, command_config, cluster, namespace):
    if output is not None:
        if resource_exists("kafkatopics", topic, cluster, namespace):
            os.system(
                Kubectl().get().kafkatopics(topic).namespace(namespace).output(output).build())
    else:
        if native:
            native_command = "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:{port} --describe " \
                             "--topic {topic}"
            pod = cluster + "-kafka-0"
            container = "kafka"
            if command_config is not None:
                native_command = apply_client_config_from_file(native_command, command_config, "--command-config",
                                                               container, pod, namespace)
            os.system(
                Kubectl().exec("-it", pod).container(container).namespace(namespace).exec_command(
                    native_command).build().format(port=KAFKA_PORT, topic=topic, cluster=cluster))
        else:
            if resource_exists("kafkatopics", topic, cluster, namespace):
                os.system(
                    Kubectl().describe().kafkatopics(topic).namespace(namespace).build())


def delete(topic, cluster, namespace):
    if resource_exists("kafkatopics", topic, cluster, namespace):
        os.system(Kubectl().delete().kafkatopics(topic).namespace(namespace).build())


def alter(topic, partitions, replication_factor, config, delete_config, cluster, namespace):
    if resource_exists("kafkatopics", topic, cluster, namespace):
        stream = get_resource_as_stream("kafkatopics", topic, namespace)
        topic_dict = yaml.full_load(stream)

        if partitions is not None:
            topic_dict["spec"]["partitions"] = int(partitions)

        if replication_factor is not None:
            topic_dict["spec"]["replicas"] = int(replication_factor)

        delete_last_applied_configuration(topic_dict)

        if len(config) > 0:
            if topic_dict["spec"].get("config") is None:
                topic_dict["spec"]["config"] = {}
            add_resource_kv_config(config, topic_dict["spec"]["config"])

        if len(delete_config) > 0:
            if topic_dict["spec"].get("config") is not None:
                delete_resource_config(delete_config, topic_dict["spec"]["config"])

        topic_yaml = yaml.dump(topic_dict)
        topic_temp_file = create_temp_file(topic_yaml)
        os.system(
            Kubectl().apply().from_file("{topic_temp_file_path}").namespace(namespace).build().format(
                topic_temp_file_path=topic_temp_file.name))
        topic_temp_file.close()
    else:
        print_resource_not_found_msg(cluster, namespace)



import click
import os
import ntpath

from kfk.commands.main import kfk
from kfk.kubectl_command_builder import Kubectl
from kfk.config import *
from kfk.commons import *
from kfk.constants import *


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--from-beginning', help='Consumes messages from beginning', is_flag=True)
@click.option('--consumer.config', 'consumer_config', help='Consumer config properties file.')
@click.option('--topic', help='Topic Name', required=True)
@kfk.command()
def console_consumer(topic, consumer_config, from_beginning, cluster, namespace):
    """Reads data from Kafka topics and outputs it to standard output."""
    native_command = "bin/kafka-console-consumer.sh --bootstrap-server {cluster}-kafka-brokers:{port} --topic {" \
                     "topic} {from_beginning}"
    pod = cluster + "-kafka-0"
    container = "kafka"
    if consumer_config is not None:
        native_command = apply_client_config_from_file(native_command, consumer_config, "--consumer.config",
                                                       container, pod, namespace)
    os.system(
        Kubectl().exec("-it", pod).container(container).namespace(namespace).exec_command(
            native_command).build().format(port=KAFKA_PORT, topic=topic, cluster=cluster,
                                           from_beginning=(from_beginning and '--from-beginning' or '')))


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--producer.config', 'producer_config', help='Producer config properties file.')
@click.option('--topic', help='Topic Name', required=True)
@kfk.command()
def console_producer(topic, producer_config, cluster, namespace):
    """Reads data from standard input and publish it to Kafka."""
    native_command = "bin/kafka-console-producer.sh --broker-list {cluster}-kafka-brokers:{port} --topic {topic}"
    pod = cluster + "-kafka-0"
    container = "kafka"
    if producer_config is not None:
        native_command = apply_client_config_from_file(native_command, producer_config, "--producer.config",
                                                       container, pod, namespace)
    os.system(
        Kubectl().exec("-it", pod).container(container).namespace(namespace).exec_command(
            native_command).build().format(port=KAFKA_PORT, topic=topic, cluster=cluster))



import click
import os

from kfk.commands.main import kfk


@kfk.command()
def env():
    """Prints the environment variable values for Strimzi Kafka CLI"""
    click.echo("STRIMZI_KAFKA_CLI_BASE_PATH: {}".format(os.environ.get('STRIMZI_KAFKA_CLI_BASE_PATH')))
    click.echo("STRIMZI_KAFKA_CLI_STRIMZI_VERSION: {}".format(os.environ.get('STRIMZI_KAFKA_CLI_STRIMZI_VERSION')))
    click.echo("STRIMZI_KAFKA_CLI_STRIMZI_PATH: {}".format(os.environ.get('STRIMZI_KAFKA_CLI_STRIMZI_PATH')))
    click.echo("STRIMZI_KAFKA_CLI_KUBECTL_VERSION: {}".format(os.environ.get('STRIMZI_KAFKA_CLI_KUBECTL_VERSION')))
    click.echo("STRIMZI_KAFKA_CLI_KUBECTL_PATH: {}".format(os.environ.get('STRIMZI_KAFKA_CLI_KUBECTL_PATH')))



import click
import os
import yaml

from kfk.commands.main import kfk
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.commons import *
from kfk.config import *
from kfk.kubectl_command_builder import Kubectl
from kfk.utils import snake_to_camel_case


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--delete-quota', 'delete_quota_tuple', help='User quotas to be removed.', multiple=True)
@click.option('--quota', 'quota_tuple', help='User\'s network and CPU utilization quotas in the Kafka cluster.',
              multiple=True)
@click.option('--resource-pattern-type',
              help="The type of the resource pattern or <ANY|MATCH|LITERAL|PREFIXED> pattern filter. When adding "
                   "acls, this should be a specific pattern type, e.g. 'literal' or 'prefixed'. (default: literal)",
              default='literal')
@click.option('--resource-name', help='ACL resource name.', cls=RequiredIf, required_if=['add_acl', 'delete_acl'])
@click.option('--resource-type', help='ACL resource type.',
              type=click.Choice(['topic', 'group', 'cluster'], case_sensitive=True), cls=RequiredIf,
              required_if=['add_acl', 'delete_acl'])
@click.option('--type', help='Operation type for ACL. (default: allow)',
              type=click.Choice(['allow', 'deny'], case_sensitive=True), default='allow')
@click.option('--host', help='Host which User will have access. (default: *)', default='*')
@click.option('--operation', 'operation_tuple', help='Operation that is being allowed or denied. (default: All)',
              default=["All"], multiple=True)
@click.option('--delete-acl', help='Delete ACL of User', is_flag=True)
@click.option('--add-acl', help='Add ACL to User', is_flag=True)
@click.option('--authorization-type', help='Authorization type for user',
              type=click.Choice(['none', 'simple'], case_sensitive=True))
@click.option('--alter', 'is_alter', help='Alter authentication-type, quotas, etc. of the user.', is_flag=True)
@click.option('--delete', 'is_delete', help='Delete a user.', is_flag=True)
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--describe', 'is_describe', help='List details for the given user.', is_flag=True)
@click.option('--authentication-type', help='Authentication type for user',
              type=click.Choice(['tls', 'scram-sha-512'], case_sensitive=True), cls=RequiredIf,
              required_if=['is_create'])
@click.option('--create', 'is_create', help='Create a new user.', is_flag=True)
@click.option('--list', 'is_list', help='List all available users.', is_flag=True)
@click.option('--user', help='User Name', required=True, cls=NotRequiredIf, not_required_if=['is_list'])
@kfk.command()
def users(user, is_list, is_create, authentication_type, is_describe, output, is_delete, is_alter, authorization_type,
          add_acl, delete_acl, operation_tuple, host, type, resource_type, resource_name, resource_pattern_type,
          quota_tuple, delete_quota_tuple, cluster, namespace):
    """Creates, alters, deletes, describes Kafka users(s)."""
    if is_list:
        list(cluster, namespace)
    elif is_create:
        create(user, authentication_type, quota_tuple, cluster, namespace)
    elif is_describe:
        describe(user, output, cluster, namespace)
    elif is_delete:
        delete(cluster, namespace, user)
    elif is_alter:
        alter(user, authentication_type, authorization_type, add_acl, delete_acl, operation_tuple, host, type,
              resource_type, resource_name, resource_pattern_type, quota_tuple, delete_quota_tuple, cluster, namespace)
    else:
        print_missing_options_for_command("users")


def list(cluster, namespace):
    os.system(
        Kubectl().get().kafkausers().label("strimzi.io/cluster={cluster}").namespace(namespace).build().format(
            cluster=cluster))


def create(user, authentication_type, quota, cluster, namespace):
    with open('{strimzi_path}/examples/user/kafka-user.yaml'.format(strimzi_path=STRIMZI_PATH).format(
            version=STRIMZI_VERSION)) as file:
        user_dict = yaml.full_load(file)

        user_dict["metadata"]["name"] = user
        user_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        user_dict["spec"]["authentication"]["type"] = authentication_type
        del user_dict["spec"]["authorization"]

        if len(quota) > 0:
            if user_dict["spec"].get("quotas") is None:
                user_dict["spec"]["quotas"] = {}
            add_resource_kv_config(quota, user_dict["spec"]["quotas"])

        user_yaml = yaml.dump(user_dict)
        user_temp_file = create_temp_file(user_yaml)
        os.system(
            Kubectl().create().from_file("{user_temp_file_path}").namespace(namespace).build().format(
                user_temp_file_path=user_temp_file.name))
        user_temp_file.close()


def describe(user, output, cluster, namespace):
    if output is not None:
        if resource_exists("kafkausers", user, cluster, namespace):
            os.system(Kubectl().get().kafkausers(user).namespace(namespace).output(output).build())
    else:
        if resource_exists("kafkausers", user, cluster, namespace):
            os.system(Kubectl().describe().kafkausers(user).namespace(namespace).build())


def delete(cluster, namespace, user):
    if resource_exists("kafkausers", user, cluster, namespace):
        os.system(Kubectl().delete().kafkausers(user).namespace(namespace).build())


def alter(user, authentication_type, authorization_type, add_acl, delete_acl, operation_tuple, host, type,
          resource_type, resource_name, resource_pattern_type, quota_tuple, delete_quota_tuple, cluster, namespace):
    if resource_exists("kafkausers", user, cluster, namespace):
        stream = get_resource_as_stream("kafkausers", user, namespace)
        user_dict = yaml.full_load(stream)

        if authentication_type is not None:
            if user_dict["spec"].get("authentication") is None:
                user_dict["spec"]["authentication"] = {}
            user_dict["spec"]["authentication"]["type"] = authentication_type

        if authorization_type is not None:
            if user_dict["spec"].get("authorization") is None:
                user_dict["spec"]["authorization"] = {}
            if authorization_type != "none":
                user_dict["spec"]["authorization"]["type"] = authorization_type
            else:
                del user_dict["spec"]["authorization"]

        if add_acl:
            if user_dict["spec"].get("authorization") is not None:
                _add_acl_option(user_dict, operation_tuple, host, type, resource_type, resource_name, resource_pattern_type)

        if delete_acl:
            if user_dict["spec"].get("authorization") is not None:
                _delete_acl_option(user_dict, operation_tuple, host, type, resource_type, resource_name,
                                   resource_pattern_type)

        delete_last_applied_configuration(user_dict)

        if len(quota_tuple) > 0:
            if user_dict["spec"].get("quotas") is None:
                user_dict["spec"]["quotas"] = {}
            add_resource_kv_config(quota_tuple, user_dict["spec"]["quotas"], snake_to_camel_case)

        if len(delete_quota_tuple) > 0:
            if user_dict["spec"].get("quotas") is not None:
                delete_resource_config(delete_quota_tuple, user_dict["spec"]["quotas"], snake_to_camel_case)

        user_yaml = yaml.dump(user_dict)
        user_temp_file = create_temp_file(user_yaml)
        os.system(
            Kubectl().apply().from_file("{user_temp_file_path}").namespace(namespace).build().format(
                user_temp_file_path=user_temp_file.name))
        user_temp_file.close()
    else:
        print_resource_not_found_msg(cluster, namespace)


def _add_acl_option(user_dict, operation, host, type, resource_type, resource_name, resource_pattern_type):
    if user_dict["spec"]["authorization"].get("acls") is None:
        user_dict["spec"]["authorization"]["acls"] = []
    for operation_str in operation:
        acl_dict = {'operation': operation_str, 'host': host, 'type': type,
                    'resource': {'type': resource_type, 'name': resource_name, 'patternType': resource_pattern_type}}
        user_dict["spec"]["authorization"]["acls"].append(acl_dict)


def _delete_acl_option(user_dict, operation, host, type, resource_type, resource_name, resource_pattern_type):
    if user_dict["spec"]["authorization"].get("acls") is None:
        user_dict["spec"]["authorization"]["acls"] = []
    for operation_str in operation:
        for acl_dict in user_dict["spec"]["authorization"]["acls"]:
            acl_dict_to_be_deleted = {'operation': operation_str, 'host': host, 'type': type,
                                      'resource': {'type': resource_type, 'name': resource_name,
                                                   'patternType': resource_pattern_type}}
            if acl_dict == acl_dict_to_be_deleted:
                user_dict["spec"]["authorization"]["acls"].remove(acl_dict)



import click


class NotRequiredIf(click.Option):

    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.not_required_if = kwargs.pop('not_required_if')
        assert self.not_required_if, "'not_required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
                          ' This argument is mutually exclusive with %s' %
                          self.not_required_if
                          ).strip()
        super(NotRequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        options_exist = self.name in opts
        control_options_exist = False

        for not_required_if in self.not_required_if:
            control_options_exist = not_required_if in opts
            if control_options_exist is True:
                break

        if control_options_exist:
            if not options_exist:
                self.required = None

        return super(NotRequiredIf, self).handle_parse_result(
            ctx, opts, args)


class RequiredIf(click.Option):

    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.required_if = kwargs.pop('required_if')
        assert self.required_if, "'required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
                          ' This argument is mutually inclusive with %s' %
                          self.required_if
                          ).strip()
        super(RequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        options_exist = self.name in opts
        control_options_exist = False

        for required_if in self.required_if:
            control_options_exist = required_if in opts
            if control_options_exist is True:
                break

        if control_options_exist or options_exist:
            self.required = True
        else:
            self.required = None

        return super(RequiredIf, self).handle_parse_result(
            ctx, opts, args)



import click
import os
import yaml

from kfk.commands.main import kfk
from kfk.kubectl_command_builder import Kubectl
from kfk.commands import topics
from kfk.commands import users
from kfk.commands import clusters
from kfk.commons import *
from kfk.constants import *
from kfk.messages import Messages
from kfk.option_extensions import NotRequiredIf


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--delete-config', help='Config keys to remove')
@click.option('--add-config', help='Key Value pairs of configs to add.')
@click.option('--alter', help='Alter the configuration for the entity.', is_flag=True)
@click.option('--native', help='List configs for the given entity natively.', is_flag=True)
@click.option('--describe', help='List configs for the given entity.', is_flag=True)
@click.option('--entity-name', help='Name of entity', required=True, cls=NotRequiredIf, not_required_if=['native'])
@click.option('--entity-type', help='Type of entity (topics/users/brokers)',
              type=click.Choice(['topics', 'users', 'brokers'], case_sensitive=True))
@kfk.command()
def configs(entity_type, entity_name, describe, native, alter, add_config, delete_config, cluster, namespace):
    """Adds/Removes entity config for a topic, client, user or brokers"""
    if describe:
        if entity_type == "topics":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
            else:
                topics.describe(entity_name, None, False, None, cluster, namespace)
        elif entity_type == "users":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
            else:
                users.describe(entity_name, None, cluster, namespace)
        elif entity_type == "brokers":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
                stream = get_resource_as_stream("configmap", cluster + "-kafka-config", namespace)
                config_dict = yaml.full_load(stream)
                config_data = get_list_by_split_string(config_dict["data"]["server.config"],
                                                       SpecialTexts.BROKER_CONFIG_FILE_USER_CONFIG_HEADER)[1]
                click.echo(NEW_LINE + Messages.USER_PROVIDED_CONFIG_HEADER + NEW_LINE + config_data)
            else:
                clusters.describe(cluster, None, namespace)

    elif alter:
        add_config_list = get_config_list(add_config)
        delete_config_list = get_config_list(delete_config)

        if entity_type == "topics":
            topics.alter(entity_name, None, None, add_config_list, delete_config_list, cluster, namespace)
        elif entity_type == "users":
            users.alter(entity_name, None, None, False, False, tuple(), None, None, None, None, None,
                                add_config_list, delete_config_list, cluster, namespace)
        elif entity_type == "brokers":
            clusters.alter(entity_name, add_config_list, delete_config_list, namespace)
    else:
        print_missing_options_for_command("configs")


def _describe_natively(entity_type, entity_name, cluster, namespace):
    native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:{port} " \
                     "--entity-type {entity_type} --describe"

    if entity_name is not None:
        native_command = native_command + SPACE + "--entity-name {entity_name}"

        if entity_type == "users":
            entity_name = COMMON_NAME_PREFIX + entity_name

    os.system(
        Kubectl().exec("-it", cluster + "-kafka-0").container("kafka").namespace(namespace).exec_command(
            native_command).build().format(cluster=cluster, port=KAFKA_PORT, entity_type=entity_type,
                                           entity_name=entity_name))



import click
import os
import yaml

from kfk.commands.main import kfk
from kfk.commons import *
from kfk.config import *
from kfk.messages import Errors
from kfk.constants import *
from jproperties import Properties

CONNECT_SKIPPED_PROPERTIES = (
    SpecialTexts.CONNECT_BOOTSTRAP_SERVERS, SpecialTexts.CONNECT_OUTPUT_IMAGE, SpecialTexts.CONNECT_PLUGIN_URL,
    SpecialTexts.CONNECT_PLUGIN_PATH)


# TODO enter command explanations
@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.argument('config_files', nargs=-1, type=click.File('r'))
@click.option('--replica-count', help='', type=int)
@click.option('--cluster', help='Connect cluster name')
@click.option('--create', 'is_create', help='Create a new connect cluster.', is_flag=True)
@kfk.command()
def connect(is_create, cluster, replica_count, config_files, namespace):
    """Creates, alters, deletes, describes Kafka connect cluster(s) or connectors."""
    if is_create:
        create(cluster, replica_count, config_files, namespace)

    else:
        print_missing_options_for_command("connect")


def create(cluster, replica_count, config_files, namespace):
    if cluster is not None:
        if len(config_files) == 0:
            raise click.ClickException(Errors.CONFIG_FILE_SHOULD_BE_PROVIDED)

        with open('{strimzi_path}/examples/connect/kafka-connect.yaml'.format(strimzi_path=STRIMZI_PATH).format(
                version=STRIMZI_VERSION)) as file:
            cluster_dict = yaml.full_load(file)

            connect_properties = Properties()
            connect_properties.load(config_files[0].read())

            cluster_dict["metadata"]["name"] = cluster
            cluster_dict["spec"]["bootstrapServers"] = connect_properties.get(SpecialTexts.CONNECT_BOOTSTRAP_SERVERS).data
            cluster_dict["spec"]["tls"] = {}

            if replica_count is not None:
                cluster_dict["spec"]["replicas"] = replica_count

            cluster_dict["spec"]["config"] = {}

            for item in connect_properties.items():
                if item[0] not in CONNECT_SKIPPED_PROPERTIES:
                    cluster_dict["spec"]["config"][item[0]] = item[1].data

            cluster_yaml = yaml.dump(cluster_dict)
            cluster_temp_file = create_temp_file(cluster_yaml)
            os.system(
                Kubectl().create().from_file("{cluster_temp_file_path}").namespace(namespace).build().format(
                    cluster_temp_file_path=cluster_temp_file.name))
            cluster_temp_file.close()



def convert_string_to_type(str_val):
    str_val = str(str_val)
    if str_val.isdigit():
        return int(str_val)
    elif is_number(str_val):
        return float(str_val)
    elif is_bool(str_val):
        return convert_string_to_boolean(str_val)
    else:
        return str_val


def is_number(str_val):
    try:
        float(str_val)
    except ValueError:
        return False
    return True


def is_bool(str_val):
    if str_val == 'true' or str_val == 'false':
        return True
    else:
        return False


def convert_string_to_boolean(str_val):
    if str_val == 'true':
        return True
    else:
        return False


def snake_to_camel_case(snake_str):
    components = snake_str.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


def get_list_by_split_string(str_val, split_char):
    # TODO: exception here
    return str_val.split(split_char)




SPACE = " "
COLON = ":"
SEMICOLON = ";"
AMPERSAND = "&"
NEW_LINE = "\n"
KAFKA_PORT = "9092"
KAFKA_SECURE_PORT = "9093"
KAFKA_SSL = "SSL"
COMMON_NAME_PREFIX = "CN="
BROKER_TMP_FOLDER_PATH = "/tmp"


class SpecialTexts:
    BROKER_CONFIG_FILE_USER_CONFIG_HEADER = "\n##########\n# User provided configuration\n##########\n"
    CONNECT_BOOTSTRAP_SERVERS = "bootstrap.servers"
    CONNECT_OUTPUT_IMAGE = "output.image"
    CONNECT_PLUGIN_URL = "plugin.url"
    CONNECT_PLUGIN_PATH = "plugin.path"



import os
import subprocess
import stat
import wget
import tarfile
import ssl

from kfk.config import *
from pathlib import Path
from kfk.kubectl_command_builder import Kubectl

ssl._create_default_https_context = ssl._create_unverified_context


def setup():
    download_kubectl_if_not_exists()
    update_kubectl_if_new_version_exists()
    download_strimzi_if_not_exists()


def download_kubectl_if_not_exists():
    if not os.path.exists(KUBECTL_PATH):
        print("Creating Strimzi Kafka CLI Dependencies folder: {base_path}\n".format(base_path=BASE_PATH))
        Path(BASE_PATH).mkdir(exist_ok=True)

        _download_kubectl()


def update_kubectl_if_new_version_exists():
    if os.path.exists(KUBECTL_PATH) and os.environ.get('STRIMZI_KAFKA_CLI_KUBECTL_VERSION') is None and os.environ.get(
            'STRIMZI_KAFKA_CLI_KUBECTL_PATH') is None and KUBECTL_VERSION not in subprocess.check_output(
            Kubectl().version("--client=true").build() + "; exit 0", shell=True, stderr=subprocess.STDOUT).decode(
            "utf-8"):
        os.rename(KUBECTL_PATH, KUBECTL_PATH + "_old")
        _download_kubectl()


def _download_kubectl():
    print("Downloading dependency: kubectl {version}...\n".format(version=KUBECTL_VERSION))
    wget.download(KUBECTL_RELEASE_URL, KUBECTL_PATH)
    print("\nDownload successfully completed!\n")
    current_stat = os.stat(KUBECTL_PATH)
    os.chmod(KUBECTL_PATH, current_stat.st_mode | stat.S_IEXEC)


def download_strimzi_if_not_exists():
    if not os.path.exists(STRIMZI_PATH):
        strimzi_tarfile_path = STRIMZI_PATH + ".tar.gz"

        print(
            "Creating Strimzi Kafka CLI Dependencies folder: {base_path}\n".format(base_path=BASE_PATH))
        Path(BASE_PATH).mkdir(exist_ok=True)

        print("Downloading dependency: Strimzi {version}...\n".format(version=STRIMZI_VERSION))
        wget.download(STRIMZI_RELEASE_URL, strimzi_tarfile_path)
        print("\nDownload successfully completed!\n")
        print("Extracting Strimzi {version}...\n".format(version=STRIMZI_VERSION))
        tar = tarfile.open(strimzi_tarfile_path)
        tar.extractall(path=BASE_PATH)
        tar.close()
        os.remove(strimzi_tarfile_path)



import platform
import sys
import os

from pathlib import Path


def _get_processor_type():
    if _is_64_bit():
        if "arm" in os.uname().machine or "aarch" in os.uname().machine:
            return "arm64"
        else:
            return "amd64"
    else:
        if "arm" in os.uname().machine:
            return "arm"
        else:
            return "386"


def _is_64_bit():
    return sys.maxsize > 2 ** 32


STRIMZI_VERSION = "0.22.1"
KUBECTL_VERSION = "v1.20.5"

BASE_FOLDER = ".strimzi-kafka-cli"
BASE_PATH = (str(Path.home()) + "/" + BASE_FOLDER) if os.environ.get(
    'STRIMZI_KAFKA_CLI_BASE_PATH') is None else os.environ.get('STRIMZI_KAFKA_CLI_BASE_PATH')
STRIMZI_VERSION = STRIMZI_VERSION if os.environ.get('STRIMZI_KAFKA_CLI_STRIMZI_VERSION') is None else os.environ.get(
    'STRIMZI_KAFKA_CLI_STRIMZI_VERSION')
STRIMZI_PATH = (BASE_PATH + "/strimzi-{version}".format(version=STRIMZI_VERSION)) if os.environ.get(
    'STRIMZI_KAFKA_CLI_STRIMZI_PATH') is None else os.environ.get('STRIMZI_KAFKA_CLI_STRIMZI_PATH')
STRIMZI_RELEASE_URL = "https://github.com/strimzi/strimzi-kafka-operator/releases/download/{version}/strimzi-{version}.tar.gz".format(
    version=STRIMZI_VERSION)
KUBECTL_VERSION = KUBECTL_VERSION if os.environ.get('STRIMZI_KAFKA_CLI_KUBECTL_VERSION') is None else os.environ.get(
    'STRIMZI_KAFKA_CLI_KUBECTL_VERSION')
KUBECTL = "kubectl" if platform.system().lower() != "windows" else "kubectl.exe"
KUBECTL_PATH = (BASE_PATH + "/" + KUBECTL) if os.environ.get(
    'STRIMZI_KAFKA_CLI_KUBECTL_PATH') is None else os.environ.get('STRIMZI_KAFKA_CLI_KUBECTL_PATH')
PROCESSOR_TYPE = _get_processor_type()
KUBECTL_RELEASE_URL = "https://storage.googleapis.com/kubernetes-release/release/{version}/bin/{operating_system}/{processor_type}/{kubectl}".format(
    version=KUBECTL_VERSION, operating_system=platform.system().lower(), processor_type=PROCESSOR_TYPE, kubectl=KUBECTL)



import click
import pkg_resources

from kfk.commands.main import kfk
from kfk.config import KUBECTL_VERSION, STRIMZI_VERSION


@kfk.command()
def version():
    """Prints the version of Strimzi Kafka CLI"""
    version = pkg_resources.require("strimzi-kafka-cli")[0].version
    click.echo("CLI Version: {}".format(version))
    click.echo("Strimzi Version: {}".format(STRIMZI_VERSION))
    click.echo("Kubectl Version: {}".format(KUBECTL_VERSION))
